/*
 * Name:    i_phidget_old_stepper.c
 *
 * Purpose: MX interface driver for the old non-HID Phidget stepper motor
 *          controller.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2004 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_types.h"
#include "mx_usb.h"

#include "i_phidget_old_stepper.h"

MX_RECORD_FUNCTION_LIST mxi_phidget_old_stepper_record_function_list = {
	NULL,
	mxi_phidget_old_stepper_create_record_structures,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	mxi_phidget_old_stepper_open,
	NULL,
	NULL,
	mxi_phidget_old_stepper_open
};

MX_RECORD_FIELD_DEFAULTS mxi_phidget_old_stepper_rf_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MXI_PHIDGET_OLD_STEPPER_STANDARD_FIELDS
};

long mxi_phidget_old_stepper_num_record_fields
		= sizeof( mxi_phidget_old_stepper_rf_defaults )
		/ sizeof( mxi_phidget_old_stepper_rf_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxi_phidget_old_stepper_rfield_def_ptr
			= &mxi_phidget_old_stepper_rf_defaults[0];

/* --- */

static mx_status_type
mxi_phidget_old_stepper_get_pointers(
	    MX_RECORD *phidget_old_stepper_controller_record,
	    MX_PHIDGET_OLD_STEPPER_CONTROLLER **phidget_old_stepper_controller,
	    MX_USB **usb,
	    const char *calling_fname )
{
	static const char fname[] = "mxi_phidget_old_stepper_get_pointers()";

	MX_PHIDGET_OLD_STEPPER_CONTROLLER *phidget_old_stepper_controller_ptr;
	MX_RECORD *usb_record;

	if ( phidget_old_stepper_controller_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
			"The MX_RECORD pointer passed by '%s' was NULL.",
			calling_fname );
	}

	phidget_old_stepper_controller_ptr =
		(MX_PHIDGET_OLD_STEPPER_CONTROLLER *)
		    phidget_old_stepper_controller_record->record_type_struct;

	if ( phidget_old_stepper_controller_ptr ==
			(MX_PHIDGET_OLD_STEPPER_CONTROLLER *) NULL )
	{
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
    "The MX_PHIDGET_OLD_STEPPER_CONTROLLER pointer for record '%s' is NULL.",
			phidget_old_stepper_controller_record->name );
	}

	if ( phidget_old_stepper_controller != NULL ) {
		*phidget_old_stepper_controller
			= phidget_old_stepper_controller_ptr;
	}

	if ( usb != (MX_USB **) NULL ) {
		usb_record = phidget_old_stepper_controller_ptr->usb_record;

		if ( usb_record == (MX_RECORD *) NULL ) {
			return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"The usb_record pointer for record '%s' is NULL.",
				phidget_old_stepper_controller_record->name );
		}

		*usb = (MX_USB *) usb_record->record_class_struct;

		if ( (*usb) == (MX_USB *) NULL ) {
			return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
	"The MX_USB pointer for USB record '%s' used by record '%s' is NULL.",
				usb_record->name,
				phidget_old_stepper_controller_record->name );
		}
	}

	return MX_SUCCESSFUL_RESULT;
}

/* ====== */

MX_EXPORT mx_status_type
mxi_phidget_old_stepper_create_record_structures( MX_RECORD *record )
{
	static const char fname[] =
			"mxi_phidget_old_stepper_create_record_structures()";

	MX_PHIDGET_OLD_STEPPER_CONTROLLER *phidget_old_stepper_controller;

	phidget_old_stepper_controller = (MX_PHIDGET_OLD_STEPPER_CONTROLLER *)
			malloc( sizeof(MX_PHIDGET_OLD_STEPPER_CONTROLLER) );

	if ( phidget_old_stepper_controller ==
			(MX_PHIDGET_OLD_STEPPER_CONTROLLER *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
    "Can't allocate memory for MX_PHIDGET_OLD_STEPPER_CONTROLLER structure." );
	}

	record->record_class_struct = NULL;
	record->class_specific_function_list = NULL;

	record->record_type_struct = phidget_old_stepper_controller;

	phidget_old_stepper_controller->record = record;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_phidget_old_stepper_open( MX_RECORD *record )
{
	static const char fname[] = "mxi_phidget_old_stepper_open()";

	MX_PHIDGET_OLD_STEPPER_CONTROLLER *phidget_old_stepper_controller;
	MX_USB *usb;
	MX_USB_DEVICE *usb_device, *usb_bootloader_device;
	mx_status_type mx_status;

	mx_status = mxi_phidget_old_stepper_get_pointers( record,
						&phidget_old_stepper_controller,
						&usb, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	MX_DEBUG(-2,("%s invoked for record '%s'.", fname, record->name));

	phidget_old_stepper_controller->usb_device = NULL;

	/* See if the firmware has already been uploaded by checking for the
	 * presence of the primary device (product ID 0x0046).
	 */

	mx_status = mx_usb_find_device_by_order( usb, &usb_device,
				MX_PHIDGET_OLD_STEPPER_VENDOR_ID,
				MX_PHIDGET_OLD_STEPPER_PRODUCT_ID,
				phidget_old_stepper_controller->order_number );

	/* If we found the primary device, we are done now. */

	if ( mx_status.code == MXE_SUCCESS ) {
		phidget_old_stepper_controller->usb_device = usb_device;

		return mx_status;
	}

	/* If we got any error code other than MXE_NOT_FOUND, then we
	 * return the error code to the caller.
	 */

	if ( mx_status.code != MXE_SUCCESS ) {
		if ( mx_status.code != MXE_NOT_FOUND ) {
			return mx_status;
		}
	}

	/* If the stepper firmware has not already been uploaded,
	 * try to upload it now.
	 */

	/* Look for the bootloader device (product ID 0x0060). */

	mx_status = mx_usb_find_device_by_order( usb,
				&usb_bootloader_device,
				MX_PHIDGET_OLD_STEPPER_VENDOR_ID,
				MX_PHIDGET_OLD_STEPPER_BOOTLOADER_PRODUCT_ID,
				phidget_old_stepper_controller->order_number );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	/* Upload the firmware. */

	/* Now look for the primary device again. */

	mx_status = mx_usb_find_device_by_order( usb, &usb_device,
				MX_PHIDGET_OLD_STEPPER_VENDOR_ID,
				MX_PHIDGET_OLD_STEPPER_PRODUCT_ID,
				phidget_old_stepper_controller->order_number );

	if ( mx_status.code == MXE_SUCCESS ) {
		phidget_old_stepper_controller->usb_device = usb_device;
	}

	MX_DEBUG(-2,("%s complete.", fname));

	return mx_status;
}

