/*
 * Name:    mx_mc_analog_input.h
 *
 * Purpose: MX header file for multichannel analog input devices.
 *
 * Author:  William Lavender
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 2004 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __MX_MULTICHANNEL_ANALOG_INPUT_H__
#define __MX_MULTICHANNEL_ANALOG_INPUT_H__

#include "mx_record.h"

#define MXT_MCAI_PERFORM_TIME_NORMALIZATION	0x1

typedef struct {
	MX_RECORD *record;

	long maximum_num_channels;
	long current_num_channels;

	double *channel_array;

	double scale;
	double offset;
	char units[MXU_UNITS_NAME_LENGTH+1];
	unsigned long mcai_flags;
} MX_MULTICHANNEL_ANALOG_INPUT;

#define MXLV_MCAI_CHANNEL_ARRAY		10001
#define MXLV_MCAI_MAXIMUM_NUM_CHANNELS	10002
#define MXLV_MCAI_CURRENT_NUM_CHANNELS	10002

#define MX_MULTICHANNEL_ANALOG_INPUT_STANDARD_FIELDS \
  {MXLV_MCAI_MAXIMUM_NUM_CHANNELS, -1, "maximum_num_channels", \
	  		MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, \
		offsetof(MX_MULTICHANNEL_ANALOG_INPUT, maximum_num_channels), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY),}, \
  \
  {MXLV_MCAI_CURRENT_NUM_CHANNELS, -1, "current_num_channels", \
	  		MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, \
		offsetof(MX_MULTICHANNEL_ANALOG_INPUT, current_num_channels), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY),}, \
  \
  {MXLV_MCAI_CHANNEL_ARRAY, -1, "channel_array", \
	  		MXFT_DOUBLE, NULL, 1, {MXU_VARARGS_LENGTH}, \
	MXF_REC_CLASS_STRUCT, \
			offsetof(MX_MULTICHANNEL_ANALOG_INPUT, channel_array), \
	{sizeof(double)}, NULL, MXFF_VARARGS}, \
  \
  {-1, -1, "scale", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MULTICHANNEL_ANALOG_INPUT, scale), \
	{0}, NULL, MXFF_IN_DESCRIPTION}, \
  \
  {-1, -1, "offset", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MULTICHANNEL_ANALOG_INPUT, offset), \
	{0}, NULL, MXFF_IN_DESCRIPTION}, \
  \
  {-1, -1, "units", MXFT_STRING, NULL, 1, {MXU_UNITS_NAME_LENGTH}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MULTICHANNEL_ANALOG_INPUT, units), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "mcai_flags", MXFT_HEX, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, \
		offsetof(MX_MULTICHANNEL_ANALOG_INPUT, mcai_flags), \
	{0}, NULL, MXFF_IN_DESCRIPTION}

/*
 * The structure type MX_MULTICHANNEL_ANALOG_INPUT_FUNCTION_LIST contains
 * a list of pointers to functions that vary from driver to driver.
 */

typedef struct {
	mx_status_type ( *read ) ( MX_MULTICHANNEL_ANALOG_INPUT *mcai );
} MX_MULTICHANNEL_ANALOG_INPUT_FUNCTION_LIST;

MX_API mx_status_type mx_multichannel_analog_input_initialize_type(
			long record_type,
			long *num_record_fields,
			MX_RECORD_FIELD_DEFAULTS **record_field_defaults,
			long *maximum_num_channels_varargs_cookie );

MX_API mx_status_type mx_multichannel_analog_input_finish_record_initialization(
							MX_RECORD *mcai_record);

MX_API mx_status_type mx_multichannel_analog_input_read( MX_RECORD *mcai_record,
						unsigned long *num_channels,
						double **channel_array );

#endif /* __MX_MULTICHANNEL_ANALOG_INPUT_H__ */

